package eu.maiora.db

import eu.maiora.model.*
import kotlinx.coroutines.Dispatchers
import kotlinx.datetime.toKotlinLocalDateTime
import org.jetbrains.exposed.dao.IntEntity
import org.jetbrains.exposed.dao.IntEntityClass
import org.jetbrains.exposed.dao.LongEntity
import org.jetbrains.exposed.dao.LongEntityClass
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.IdTable
import org.jetbrains.exposed.sql.Transaction
import org.jetbrains.exposed.sql.javatime.date
import org.jetbrains.exposed.sql.javatime.datetime
import org.jetbrains.exposed.sql.transactions.experimental.newSuspendedTransaction
import java.time.format.DateTimeFormatter



object AccountsTable : IdTable<Int>("accounts"){
    override val id = integer("id").entityId()
    val username = varchar("username", 255)
    val password = varchar("password", 255)

    override val primaryKey = PrimaryKey(id)
}

object ParametriTable : IdTable<Int>("parametri"){
    override val id = integer("id").entityId()
    val chiave = varchar("chiave", 255)
    val valore = varchar("valore", 255)

    override val primaryKey = PrimaryKey(id)
}

object TessereTable : IdTable<Long>("view_tessere_api"){
    override val id = long("id").entityId()
    val idUtente = long("id_utente")
    val codiceFiscale = varchar("codice_fiscale", 255)
    val numero = varchar("numero", 255)
    val saldo = double("saldo")
    val punti = integer("punti")

    override val primaryKey = PrimaryKey(id)
}

object MovimentiTable : IdTable<Long>("view_movimenti_api"){
    override val id = long("id").entityId()
    val idTessera = long("id_tessera")
    val numero = varchar("numero_tessera", 255)
    val operazione = varchar("operazione", 255)
    val dataMovimento = datetime("data_movimento")
    val transazione = varchar("transazione", 255)
    val dispositivo = varchar("dispositivo", 255)
    val riferimento= varchar("riferimento", 255)
    val saldoPre = double("saldo_pre")
    val importo = double("importo")
    val saldoPost = double("saldo_post")
    val puntiPre = integer("punti_pre")
    val punti = integer("punti")
    val puntiPost = integer("punti_post")
}

object ComposizioniTable : IdTable<Long>("view_composizioni"){
    override val id = long("id").entityId()
    val idVassoio = long("id_vassoio")
    val vassoio = varchar("vassoio", 255)
    val idCategoria = long("id_categoria")
    val categoria = varchar("categoria", 255)
    val quantita = integer("quantita")
}

object RicaricheTable : IdTable<Long>("ricariche_app"){
    override val id = long("id").autoIncrement("seq_ricariche_app").entityId()
    val codiceTransazione = varchar("codice_transazione", 255)
    val messaggio = varchar("messaggio", 255).nullable()
    val idTessera = long("id_tessera")
    val importo = double("importo")
}

object SlotPrenotabiliTable : IdTable<Long>("view_slot_prenotabili"){
    override val id = long("id").entityId()
    val giorno = date("giorno")
    val dataInizio = datetime("data_inizio")
    val dataFine = datetime("data_fine")
    val dataLimiteCancellazione = datetime("data_limite_cancellazione")
    val idModalita = long("id_modalita")
    val modalitaPrenotazione = varchar("modalita_prenotazione", 255)
    val idPuntoDistribuzione = long("id_punto_distribuzione")
    val puntoDistribuzione = varchar("punto_distribuzione", 255)
    val idPuntoCassa = long("id_punto_cassa")
    val puntoCassa = varchar("punto_cassa", 255)
    val idTurno = long("id_turno")
    val turno = varchar("turno", 255)
    val idResidenza = long("id_residenza").nullable()
    val residenza = varchar("residenza", 255).nullable()
}

object ProdottiPrenotabiliTable : IdTable<Long>("view_prodotti_prenotabili"){
    override val id = long("id").entityId()
    val giorno = date("giorno")
    val idPuntoDistribuzione = long("id_punto_distribuzione")
    val puntoDistribuzione = varchar("punto_distribuzione", 255)
    val idTurno = long("id_turno")
    val turno = varchar("turno", 255)
    val idProdotto = long("id_prodotto")
    val prodotto = varchar("prodotto", 255)
    val codiceProdotto = varchar("codice_prodotto", 255)
    val descrizioneProdotto = varchar("descrizione_prodotto", 255)
    val idCategoria = long("id_categoria")
    val categoria = varchar("categoria", 255)
    val idModalita = long("id_modalita")
    val modalitaPrenotazione = varchar("modalita_prenotazione", 255)
}

object ResidenzeTable : IdTable<Long>("residenze"){
    override val id = long("id").entityId()
    val idPuntoDistribuzione = long("id_punto_distribuzione")
    val nome = varchar("nome", 255)
}

object PrenotazioniPastiTable : IdTable<Long>("prenotazioni_pasti"){
    override val id = long("id").autoIncrement("seq_prenotazioni_pasti").entityId()
    val idTessera = long("id_tessera")
    val idSlotPuntoCassa = long("id_slot_punto_cassa")
    val idResidenza = long("id_residenza").nullable()
    val idVassoio = long("id_vassoio")
    val importoPagato = double("importo_pagato")
    val codiceRistocloud = varchar("codice_ristocloud", 255).nullable()
}

object PrenotazioniPastiDettaglioTable : IdTable<Long>("prenotazioni_pasti_dettaglio"){
    override val id = long("id").autoIncrement("seq_pren_past_dettaglio").entityId()
    val idPrenotazione = long("id_prenotazione")
    val idProdotto = long("id_prodotto")
    val idStato = long("id_stato")

}

object ViewPrenotazioniPastiTable : IdTable<Long>("view_prenotazioni_pasti"){
    override val id = long("id").entityId()
    val idPrenotazione = long("id_prenotazione")
    val idStato = long("id_stato")
    val stato = varchar("stato", 255)
    val dataLimiteCancellazione = datetime("data_limite_cancellazione")
}

class AccountsDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<AccountsDAO>(AccountsTable)

    var username by AccountsTable.username
    var password by AccountsTable.password
}

class ParametriDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<ParametriDAO>(ParametriTable)

    var chiave by ParametriTable.chiave
    var valore by ParametriTable.valore
}

class TessereDao(id: EntityID<Long>) :LongEntity(id) {
    companion object : LongEntityClass<TessereDao>(TessereTable)

    var idUtente by TessereTable.idUtente
    var codiceFiscale by TessereTable.codiceFiscale
    var numero by TessereTable.numero
    var saldo by TessereTable.saldo
    var punti by TessereTable.punti
}

class MovimentiDao(id: EntityID<Long>) :LongEntity(id) {
    companion object : LongEntityClass<MovimentiDao>(MovimentiTable)

    var idTessera by MovimentiTable.idTessera
    var numero by MovimentiTable.numero
    var operazione by MovimentiTable.operazione
    var dataMovimento by MovimentiTable.dataMovimento
    var transazione by MovimentiTable.transazione
    var dispositivo by MovimentiTable.dispositivo
    var riferimento by MovimentiTable.riferimento
    var saldoPre by MovimentiTable.saldoPre
    var importo by MovimentiTable.importo
    var saldoPost by MovimentiTable.saldoPost
    var puntiPre by MovimentiTable.puntiPre
    var punti by MovimentiTable.punti
    var puntiPost by MovimentiTable.puntiPost
}

class ComposizioniDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<ComposizioniDao>(ComposizioniTable)

    var idVassoio by ComposizioniTable.idVassoio
    var vassoio by ComposizioniTable.vassoio
    var idCategoria by ComposizioniTable.idCategoria
    var categoria by ComposizioniTable.categoria
    var quantita by ComposizioniTable.quantita
}

class RicaricheDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<RicaricheDao>(RicaricheTable)

    var codiceTransazione by RicaricheTable.codiceTransazione
    var messaggio by RicaricheTable.messaggio
    var idTessera by RicaricheTable.idTessera
    var importo by RicaricheTable.importo
}

class SlotPrenotabiliDao(id: EntityID<Long>) :LongEntity(id) {
    companion object : LongEntityClass<SlotPrenotabiliDao>(SlotPrenotabiliTable)

    var giorno by SlotPrenotabiliTable.giorno
    var dataInizio by SlotPrenotabiliTable.dataInizio
    var dataFine by SlotPrenotabiliTable.dataFine
    var dataLimiteCancellazione by SlotPrenotabiliTable.dataLimiteCancellazione
    var idModalita by SlotPrenotabiliTable.idModalita
    var modalitaPrenotazione by SlotPrenotabiliTable.modalitaPrenotazione
    var idPuntoDistribuzione by SlotPrenotabiliTable.idPuntoDistribuzione
    var puntoDistribuzione by SlotPrenotabiliTable.puntoDistribuzione
    var idPuntoCassa by SlotPrenotabiliTable.idPuntoCassa
    var puntoCassa by SlotPrenotabiliTable.puntoCassa
    var idTurno by SlotPrenotabiliTable.idTurno
    var turno by SlotPrenotabiliTable.turno
    var idResidenza by SlotPrenotabiliTable.idResidenza
    var residenza by SlotPrenotabiliTable.residenza
}

class ProdottiPrenotabiliDao(id: EntityID<Long>) :LongEntity(id) {
    companion object : LongEntityClass<ProdottiPrenotabiliDao>(ProdottiPrenotabiliTable)

    var giorno by ProdottiPrenotabiliTable.giorno
    var idPuntoDistribuzione by ProdottiPrenotabiliTable.idPuntoDistribuzione
    var puntoDistribuzione by ProdottiPrenotabiliTable.puntoDistribuzione
    var idTurno by ProdottiPrenotabiliTable.idTurno
    var turno by ProdottiPrenotabiliTable.turno
    var idProdotto by ProdottiPrenotabiliTable.idProdotto
    var prodotto by ProdottiPrenotabiliTable.prodotto
    var codiceProdotto by ProdottiPrenotabiliTable.codiceProdotto
    var descrizioneProdotto by ProdottiPrenotabiliTable.descrizioneProdotto
    var idCategoria by ProdottiPrenotabiliTable.idCategoria
    var categoria by ProdottiPrenotabiliTable.categoria
    var idModalita by ProdottiPrenotabiliTable.idModalita
    var modalitaPrenotazione by ProdottiPrenotabiliTable.modalitaPrenotazione
}

class ResidenzeDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<ResidenzeDao>(ResidenzeTable)

    val idPuntoDistribuzione by ResidenzeTable.idPuntoDistribuzione
    val nome by ResidenzeTable.nome
}

class PrenotazioniPastiDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<PrenotazioniPastiDao>(PrenotazioniPastiTable)

    val idTessera by PrenotazioniPastiTable.idTessera
    val idSlotPuntoCassa by PrenotazioniPastiTable.idSlotPuntoCassa
    val idResidenza by PrenotazioniPastiTable.idResidenza
    val idVassoio by PrenotazioniPastiTable.idVassoio
    val importoPagato by PrenotazioniPastiTable.importoPagato
    val codiceRistocloud by PrenotazioniPastiTable.codiceRistocloud
}

class PrenotazioniPastiDettaglioDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<PrenotazioniPastiDettaglioDao>(PrenotazioniPastiDettaglioTable)

    val idPrenotazione by PrenotazioniPastiDettaglioTable.idPrenotazione
    val idProdotto by PrenotazioniPastiDettaglioTable.idProdotto
    val idStato by PrenotazioniPastiDettaglioTable.idStato
}

class ViewPrenotazioniPastiDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<ViewPrenotazioniPastiDao>(ViewPrenotazioniPastiTable)

    val idPrenotazione by ViewPrenotazioniPastiTable.idPrenotazione
    val idStato by ViewPrenotazioniPastiTable.idStato
    val stato by ViewPrenotazioniPastiTable.stato
    val dataLimiteCancellazione by ViewPrenotazioniPastiTable.dataLimiteCancellazione
}

fun accountsDaoToModel(dao: AccountsDAO) = Accounts(
    dao.id.value,
    dao.username,
    dao.password
)

fun parametriDaoToModel(dao: ParametriDAO) = Parametri(
    dao.id.value,
    dao.chiave,
    dao.valore
)

fun tessereDaoToModel(dao: TessereDao) = Tessere(
    dao.id.value,
    dao.idUtente,
    dao.codiceFiscale,
    dao.numero,
    dao.saldo,
    dao.punti
)

fun movimentiDaoToModel(dao: MovimentiDao) :Movimenti{
    val formatter = DateTimeFormatter.ofPattern("ddMMyyyy HH:mm")
    val formattedDate = dao.dataMovimento.format(formatter)

    return Movimenti(
        dao.id.value,
        dao.idTessera,
        dao.numero,
        dao.operazione,
        formattedDate,
        dao.transazione,
        dao.dispositivo,
        dao.riferimento,
        dao.saldoPre,
        dao.importo,
        dao.saldoPost,
        dao.puntiPre,
        dao.punti,
        dao.puntiPost
    )
}

fun composizioniDaoToModel(dao: ComposizioniDao) = Composizioni(
    dao.id.value,
    dao.idVassoio,
    dao.vassoio,
    dao.idCategoria,
    dao.categoria,
    dao.quantita
)

fun puntiDistribuzioneDaoToModel(dao: SlotPrenotabiliDao) :PuntiDistribuzione{
    return PuntiDistribuzione(
        dao.id.value,
        dao.idPuntoDistribuzione,
        dao.puntoDistribuzione,
        dao.idPuntoCassa,
        dao.puntoCassa,
        dao.idModalita,
        dao.modalitaPrenotazione
    )
}

fun slotOrariDaoToModel(dao: SlotPrenotabiliDao) :SlotOrari{
    val formatterGiorno = DateTimeFormatter.ofPattern("ddMMyyyy")
    val formatterConOra = DateTimeFormatter.ofPattern("ddMMyyyy HH:mm")
    val formattedGiorno = dao.giorno.format(formatterGiorno)
    val formattedDataInizio = dao.dataInizio.format(formatterConOra)
    val formattedDataFine = dao.dataFine.format(formatterConOra)
    val formattedDataLimiteCancellazione = dao.dataLimiteCancellazione.format(formatterConOra)

    return SlotOrari(
        dao.id.value,
        formattedGiorno,
        formattedDataInizio,
        formattedDataFine,
        formattedDataLimiteCancellazione,
        dao.idPuntoDistribuzione,
        dao.puntoDistribuzione,
        dao.idPuntoCassa,
        dao.puntoCassa,
        dao.idModalita,
        dao.modalitaPrenotazione,
        dao.idTurno,
        dao.turno,
        dao.idResidenza,
        dao.residenza
    )
}

fun prodottiPrenotabiliDaoToModel(dao: ProdottiPrenotabiliDao) :ProdottiPrenotabili{
    val formatterGiorno = DateTimeFormatter.ofPattern("ddMMyyyy")
    val formattedGiorno = dao.giorno.format(formatterGiorno)

    return ProdottiPrenotabili(
        dao.id.value,
        formattedGiorno,
        dao.idPuntoDistribuzione,
        dao.puntoDistribuzione,
        dao.idTurno,
        dao.turno,
        dao.idProdotto,
        dao.prodotto,
        dao.codiceProdotto,
        dao.descrizioneProdotto,
        dao.idCategoria,
        dao.categoria,
        dao.idModalita,
        dao.modalitaPrenotazione
    )
}

fun residenzeDaoToModel(dao: ResidenzeDao) :Residenze{
    return Residenze(
        dao.id.value,
        dao.idPuntoDistribuzione,
        dao.nome
    )
}

fun prenotazioniPastiDaoToModel(dao: PrenotazioniPastiDao) : PrenotazioniPasti{
    return PrenotazioniPasti(
        dao.id.value,
        dao.idTessera,
        dao.idSlotPuntoCassa,
        dao.idResidenza,
        dao.idVassoio,
        dao.importoPagato,
        dao.codiceRistocloud
    )
}

fun prenotazioniPastiDettaglioDaoToModel(dao: PrenotazioniPastiDettaglioDao) : PrenotazioniPastiDettaglio{
    return PrenotazioniPastiDettaglio(
        dao.id.value,
        dao.idPrenotazione,
        dao.idProdotto,
        dao.idStato
    )
}

fun viewPrenotazioniPastiDaoToModel(dao: ViewPrenotazioniPastiDao) : ViewPrenotazioniPasti{
    return ViewPrenotazioniPasti(
        dao.id.value,
        dao.idPrenotazione,
        dao.idStato,
        dao.stato,
        dao.dataLimiteCancellazione.toKotlinLocalDateTime()
    )
}


suspend fun <T> suspendTransaction(block: Transaction.() -> T): T =
    newSuspendedTransaction(Dispatchers.IO, statement = block)