package eu.maiora.model

import eu.maiora.db.*
import org.jetbrains.exposed.sql.and
import java.time.LocalDate
import java.time.format.DateTimeFormatter

class SlotOrariRepositoryImpl : SlotOrariRepository {
    override suspend fun slotOrariByGiornoIdTurnoIdPuntoDistIdMod(giorno : String,
                                                                  idTurno : Long,
                                                                  idPuntoDistribuzione : Long,
                                                                  idModalita : Long): List<SlotOrari> = suspendTransaction {
        // Cerca la lista di slot orari
        val formatter = DateTimeFormatter.ofPattern("ddMMyyyy")
        val formattedDate = LocalDate.parse(giorno, formatter)

        SlotPrenotabiliDao.find { (SlotPrenotabiliTable.giorno eq formattedDate) and
                                    (SlotPrenotabiliTable.idTurno eq idTurno) and
                (SlotPrenotabiliTable.idPuntoDistribuzione eq idPuntoDistribuzione) and
                (SlotPrenotabiliTable.idModalita eq idModalita)}
            .toList() // Restituisce la lista degli slot prenotabili da cui ricavare i punti distribuzione
            .map { slotOrariDaoToModel(it) } // Converte il DAO in un oggetto SlotOrari

    }

    override suspend fun idResidenzaByIdSlotOrario(idSlot : Long) : Long? = suspendTransaction {
        SlotPrenotabiliDao.find{ SlotPrenotabiliTable.id eq idSlot }
            .single() // Restituisce un singolo risultato o null se non trovato
            .let { slotOrariDaoToModel(it) } // Converte il DAO in un oggetto SlotOrari
            .idResidenza
    }

    override suspend fun slotOrarioById(idSlot : Long) : SlotOrari? = suspendTransaction {
        SlotPrenotabiliDao.find{ SlotPrenotabiliTable.id eq idSlot }
            .singleOrNull() // Restituisce un singolo risultato o null se non trovato
            ?.let { slotOrariDaoToModel(it) } // Converte il DAO in un oggetto SlotOrari
    }
}