package eu.maiora.routes

import eu.maiora.model.*
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.datetime.toJavaLocalDateTime
import java.time.LocalDateTime


fun Route.annullaPrenotazioni(
    viewPrenotazioniPastiRepository: ViewPrenotazioniPastiRepositoryImpl,
    prenotazioniPastiDettaglioRepository: PrenotazioniPastiDettaglioRepositoryImpl
) {
    route("/api/annullaPrenotazioni") {
        authenticate("auth-jwt") {
            patch("{idPrenotazione}") {
                //ottieni l'id della prenotazione dal percorso
                val idPrenotazione = call.parameters["idPrenotazione"]

                if (idPrenotazione == null) {
                    call.respondText("ID prenotazione non valido", status = HttpStatusCode.BadRequest)
                    return@patch
                }

                val prenotazione = viewPrenotazioniPastiRepository.prenotazioniPastiById(idPrenotazione.toLong())
                val statoPrenotazione = prenotazione.stato

                //prenotazione annullabile se lo stato non  SERVITA 2L
                if (prenotazione.idStato == 2L) {
                    call.respond(
                        HttpStatusCode.BadRequest,
                        "Prenotazione non annullata. Stato corrente della prenotazione: $statoPrenotazione"
                    )
                    return@patch
                }

                //prenotazione annullabile se la data limite per la cancellazione non  stato superato
                if (LocalDateTime.now().isAfter(prenotazione.dataLimiteCancellazione.toJavaLocalDateTime())) {
                    call.respond(
                        HttpStatusCode.BadRequest,
                        "Prenotazione non annullata: data limite cancellazione superata"
                    )
                    return@patch
                }


                val listaProdottiPrenotati =
                    prenotazioniPastiDettaglioRepository.listaDettagliByIdPrenotazione(idPrenotazione.toLong())

                //aggiorna lo stato di ogni prodotto per annullare la prenotazione
                listaProdottiPrenotati.forEach { el ->
                    val prodottoDaAggiornare = PrenotazioniPastiDettaglio(
                        el.id,
                        el.idPrenotazione,
                        el.idProdotto,
                        3L //ANNULLATA
                    )
                    prenotazioniPastiDettaglioRepository.update(prodottoDaAggiornare)
                }

                call.respond(HttpStatusCode.OK, "Prenotazione $idPrenotazione annullata")


            }
        }

    }
}