package eu.maiora.routes

import eu.maiora.model.PuntiDistribuzioneRepositoryImpl
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.response.*
import io.ktor.server.routing.*


fun Route.puntiDistribuzione(puntiDistribuzioneRepository: PuntiDistribuzioneRepositoryImpl){
    route("/api/puntiDistribuzione"){
        authenticate("auth-jwt") {
            get(){
                // Ottieni l'id del turno e il giorno dal percorso
                val idTurno = call.parameters["idTurno"]
                val giorno = call.parameters["giorno"]

                if (idTurno.isNullOrEmpty()) {
                    call.respondText("ID turno non valido o non presente nei parametri", status = HttpStatusCode.BadRequest)
                    return@get
                }
                if (giorno.isNullOrEmpty()) {
                    call.respondText("Giorno non valido o non presente nei parametri", status = HttpStatusCode.BadRequest)
                    return@get
                }

                // Cerca i punti distribuzione per giorno e turno
                val listaPuntiDistribuzione = puntiDistribuzioneRepository.puntiDistByGiornoIdTurno(giorno, idTurno.toLong())


                if (listaPuntiDistribuzione != null) {
                    call.respond(listaPuntiDistribuzione)
                } else {
                    call.respondText("Punti distribuzione non trovati", status = HttpStatusCode.NotFound)
                }
            }
        }

    }
}